#include <stdio.h>

#define ROWS 9
#define COLS 9

/* Test if the numbers are in range and not repeated in the specified cells */
int testRangeRepeated(int cells[ROWS][COLS], int rowInit, int colInit, int rowEnd, int colEnd);
/* Test if there are no repeated values in the rows */
int testRows(int cells[ROWS][COLS]);
/* Test if there are no repeated values in the columns */
int testCols(int cells[ROWS][COLS]);

int main(void) {
	/* Examples */

	/* Correct */
	int sudoku1[ROWS][COLS]={
			{3,8,1,9,7,6,5,4,2},
			{2,4,7,5,3,8,1,9,6},
			{5,6,9,2,1,4,8,7,3},
			{6,7,4,8,5,2,3,1,9},
			{1,3,5,7,4,9,6,2,8},
			{9,2,8,1,6,3,7,5,4},
			{4,1,2,6,8,5,9,3,7},
			{7,9,6,3,2,1,4,8,5},
			{8,5,3,4,9,7,2,6,1}};
	/* Not correct: in the second row, number 3 is repeated */
	int sudoku2[ROWS][COLS]={
			{3,8,1,9,7,6,5,4,2},
			{4,3,7,5,3,8,1,9,6},
			{5,6,9,2,1,4,8,7,3},
			{6,7,4,8,5,2,3,1,9},
			{1,3,5,7,4,9,6,2,8},
			{9,2,8,1,6,3,7,5,4},
			{4,1,2,6,8,5,9,3,7},
			{7,9,6,3,2,1,4,8,5},
			{8,5,3,4,9,7,2,6,1}};
	/* Not correct: in the first column, numbers are repeated */
	int sudoku3[ROWS][COLS]={
			{1,2,3,4,5,6,7,8,9},
			{1,2,3,4,5,6,7,8,9},
			{1,2,3,4,5,6,7,8,9},
			{1,2,3,4,5,6,7,8,9},
			{1,2,3,4,5,6,7,8,9},
			{1,2,3,4,5,6,7,8,9},
			{1,2,3,4,5,6,7,8,9},
			{1,2,3,4,5,6,7,8,9},
			{1,2,3,4,5,6,7,8,9}};
	/* Not correct: in the first quadrant, there are repeated values */
	int sudoku4[ROWS][COLS]={
			{1,2,3,4,5,6,7,8,9},
			{2,3,4,5,6,7,8,9,1},
			{3,4,5,6,7,8,9,1,2},
			{4,5,6,7,8,9,1,2,3},
			{5,6,7,8,9,1,2,3,4},
			{6,7,8,9,1,2,3,4,5},
			{7,8,9,1,2,3,4,5,6},
			{8,9,1,2,3,4,5,6,7},
			{9,1,2,3,4,5,6,7,8}};

	printf("\nFirst sudoku: \n");
	printf("Result --> %i", testCompleteSudoku(sudoku1));

	printf("\n\nSecond sudoku: \n");
	printf("Result --> %i", testCompleteSudoku(sudoku2));

	printf("\n\nThird sudoku: \n");
	printf("Result --> %i", testCompleteSudoku(sudoku3));

	printf("\n\nFourth sudoku: \n");
	printf("Result --> %i \n\n", testCompleteSudoku(sudoku4));


	system("pause");
	return 0;
}

int testCompleteSudoku(int cells[ROWS][COLS]) {
	int r_correct, c_correct, s_correct;
	int i, j;

	/* Test rows and columns */
	printf("Checking rows...");
	r_correct = testRows(cells);
	printf("[%i]\n", r_correct);
	printf("Checking cols...");
	c_correct = testCols(cells);
	printf("[%i]\n", c_correct);

	/* Test quadrants */
	printf("Checking quadrants...");
	s_correct = 1;
	for(i=0; i<ROWS; i+=3)
		for(j=0; j<COLS; j+=3)
			s_correct &= testRangeRepeated(cells, i, j, i+2, j+2);
	printf("[%i]\n", s_correct);

	return (r_correct & c_correct & s_correct);
}

int testRangeRepeated(int cells[ROWS][COLS], int rowInit, int colInit, int rowEnd, int colEnd) {
	int i, j;
	int correct = 1;
	int numbersCount[] = {0, 0, 0, 0, 0, 0, 0, 0, 0};

	/* Test if the numbers are in 1-9 and not repeated */
	for(i=rowInit; i<=rowEnd && correct; i++) {
		for(j=colInit; j<=colEnd && correct; j++) {
			int val = cells[i][j];

			if(val <= 0 || val >= 10) {
				/* Value out of range */
				correct = 0;
			} else {
				if(numbersCount[val-1] > 0)
					/* Repeated value */
					correct = 0;
				else
					/* Not repeated */
					numbersCount[val-1] ++;
			}

		}
	}

	return correct;
}

int testRows(int cells[ROWS][COLS]) {
	int i, j;
	int correct = 1;

	/* Test if there are no repeated values in the row */
	for(i=0; i<ROWS && correct; i++) {
		int numbersCount[] = {0, 0, 0, 0, 0, 0, 0, 0, 0};

		for(j=0; j<COLS && correct; j++) {
			int val = cells[i][j];

			if(val <= 0 || val >= 10) {
				/* Value out of range */
				correct = 0;
			} else {
				if(numbersCount[val-1] > 0)
					correct = 0;
				else
					numbersCount[val-1]++;
			}
		}
	}

	return correct;
}

int testCols(int cells[ROWS][COLS]) {
	int i, j;
	int correct = 1;

	/* Test if there are no repeated values in the columns */
	for(j=0; j<COLS && correct; j++) {
		int numbersCount[] = {0, 0, 0, 0, 0, 0, 0, 0, 0};

		for(i=0; i<ROWS && correct; i++) {
			int val = cells[i][j];

			if(val <= 0 || val >= 10) {
				/* Value out of range */
				correct = 0;
			} else {
				if(numbersCount[val-1] > 0)
					correct = 0;
				else
					numbersCount[val-1]++;
			}
		}
	}

	return correct;
}

